#include <stdio.h>
#include "globals.h"

void pit_skip(ibytes)
unsigned long ibytes;
{
    while(ibytes != 0) {
	if(getc(infp) == EOF) {
	    fprintf(stderr, "Premature EOF\n");
	    exit(1);
	}
	ibytes--;
    }
}

