#include "globals.h"
#include "../fileio/fileglob.h"

/* Unsit, no compression */
void sit_nocomp(ibytes)
unsigned long ibytes;
{
    int n;

    n = fread(out_buffer, 1, (int)ibytes, infp);
    if(n != ibytes) {
	fprintf(stderr, "Premature EOF\n");
	exit(1);
    }
}

