#include "pit.h"
#include "globals.h"
#include "masks.h"

void pit_nocomp();
void pit_huffman();

void pit_wrfile(bytes, type)
unsigned long bytes;
int type;
{
    if(bytes == 0) {
	return;
    }
    switch(type) {
    case nocomp:
	pit_nocomp(bytes);
	break;
    case huffman:
	pit_huffman(bytes);
    }
}

